<?php

/**
 * @file
 * Contains \Drupal\url_embed\Plugin\Field\FieldFormatter\LinkEmbedFormatter.
 */

namespace Drupal\url_embed\Plugin\Field\FieldFormatter;

use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\FormatterBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\url_embed\UrlEmbedHelperTrait;

/**
 * Plugin implementation of the 'url_embed' formatter.
 *
 * @FieldFormatter(
 *   id = "url_embed",
 *   label = @Translation("Embedded URL"),
 *   field_types = {
 *     "link"
 *   }
 * )
 */
class LinkEmbedFormatter extends FormatterBase {
  use UrlEmbedHelperTrait;

  /**
   * {@inheritdoc}
   */
  public static function defaultSettings() {
    return [
      'responsive' => FALSE,
    ] + parent::defaultSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state) {
    $element = [];
    $element['responsive'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable responsive wrappers'),
      '#description' => $this->t('Automatically wrap embedded iframes with a container which will allow the embedded media to scale appropriately to the size of the page.'),
      '#default_value' => $this->getSetting('responsive'),
    ];

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary() {
    return [$this->t('Responsive: @responsive', ['@responsive' => $this->getSetting('responsive') ? $this->t('enabled') : $this->t('disabled')])];
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(FieldItemListInterface $items, $langcode) {
    $element = array();

    foreach ($items as $delta => $item) {
      if ($url = $item->getUrl()->toString()) {
        try {
          if ($info = $this->urlEmbed()->getEmbed($url)) {
            $url_output = $info->getCode();
            $ratio = $info->aspectRatio;

            // Wrap the embed code in a container to make it responsive
            if ($ratio && $this->getSetting('responsive')) {
              $element[$delta] = [
                '#theme' => 'responsive_embed',
                '#ratio' => $ratio,
                '#url_output' => $url_output,
                '#attached' => [
                  'library' => [
                    'url_embed/url_embed.responsive_styles',
                  ],
                ],
              ];
            }
            else {
              $element[$delta] = [
                '#type' => 'inline_template',
                '#template' => $url_output,
              ];
            }
          }
        }
        catch (\Exception $exception) {
          watchdog_exception('url_embed', $exception);
        }
      }
    }

    return $element;
  }

}
